{% extends "header.html" %}
{% block content_root %}
<div class="jumbotron">
    <div class="container">
        <h1 class="">Controller</h1>
            <a class="btn btn-primary btn-lg" href="/download" target="_blank" role="button">Download</a>
            &nbsp;
            <a class="btn btn-warning btn-lg" href="/reset" target="_blank" role="button">Reset</a>

            <a class="btn btn-danger btn-lg" href="/shutdown" target="_blank" role="button">Shutdown</a>

    </div>
</div>

{% endblock %}
