{% extends "header.html" %}
{% block content_root %}
<div class="jumbotron">
    <div class="container">
        <h2 class="">Evaluation Metrics</h2>
    </div>
</div>
<div class="container">
        {% include "trainingEval.html" %}
        {% include "validationEval.html" %}
        <hr />
</div>

{% endblock %}
