{% extends "header.html" %}
{% block content_root %}
<div class="jumbotron">
    <div class="container">
        <h2 class="">Loss Metrics</h2>
    </div>
</div>
<div class="container">
        {% include "trainingLoss.html" %}
        {% include "validationLoss.html" %}
        <hr />
</div>
{% endblock %}
